import click
import keyring
import getpass
import datetime
import sys
import os
from source.scrapper import attempt
from tabulate import tabulate
from source.missedClassScrapper import MissedClassDates

try:
    import win32con, win32api
except ImportError:
    pass

@click.command()
@click.option('-r', '--roll', prompt='Roll Number', help='Enter the Roll Number for ERP Login.')
def attendance(roll):
    """
    Get the credentials first
    """
    password = keyring.get_password('ERP', roll)
    saved_password = True

    if password != None:
        response = attempt(roll, password)

        if not response:
            click.echo(click.style('Password modified, Enter new password.', fg='green', bold=True))
            keyring.delete_password('ERP',roll)
            password = None

    if password == None:
        password = getpass.getpass("Password : ")
        response,password = ResponseAttempt(roll,password)
        saved_password = False

    # Fetch attendance from ERP and Pretty Print it on Terminal.
    attendance_table = make_table(response)
    attendance_header = ["Subject Code","Subject Name", "Attended","Percentage"]
    show_table(attendance_table,attendance_header)

    # Fetch missed classes from ERP and Pretty Print it on Terminal.
    ans = input("Do you want to see the missed class(es) date-wise ? (y/N) ")
    if ans=='y':
        input_subject = []
        specific_class_ans = input("Do you want to see the missed class(es) of specific subject(s)? (y/N) ")
        if specific_class_ans=='y':
            input_subject = [x for x in input("Enter the Subject code(s) seperated by spaces: ").split()]

        attendance = None
        cached_yes = False

        try:
            with open(os.path.expanduser("~/.attendance_past.txt"), "r") as f:
                attendance = [line.split("\t") for line in f]
            cached_yes = input("Unless your past attendance was updated recently, would you like us to use cached data? (y/N) ")
        except Exception:
            pass

        print("Please wait, it may take a while to fetch the information...")
        
        if attendance is not None and cached_yes == 'y':
            missed_class_response = MissedClassDates(roll,password, attendance[0][0])
        else:
            missed_class_response = MissedClassDates(roll,password)
        if specific_class_ans !='y':
             missed_class_table = make_missed_class_table(missed_class_response, attendance, cached_yes)
        else : 
            missed_class_table = make_specific_sub_missed_class_table(missed_class_response,input_subject, attendance, cached_yes)
        Missed_class_headers =  ["Subject Code", "Date","Subject Name", "Attended"]
        show_table(missed_class_table,Missed_class_headers)

        
    # Store password locally if not saved already
    if not saved_password:
        ans = input("Do you want to store your password locally? (y/N) ")
        if ans=='y':
            keyring.set_password('ERP', roll, password)


def make_table(response):
    result = [[data['code'], data['name'], data['attended'] + '/' + data['total'],data['percentage']] for (code, data) in response.items()]
    return result

def make_missed_class_table(response, attendance, cached_yes):
    result = []
    if attendance is not None and cached_yes == 'y':
        result = [attendance[i] for i in range (1, len(attendance))]
    for (Date, all_data) in response.items():       
        for data in all_data:
            if data is not None:
                result.append([data['code'], data['Date'], data['subjectName'], data['attended'] + '/' + data['total']]) 

    today = datetime.date.today()
    file = os.path.expanduser("~/.attendance_past.txt")

    try:
        win32api.SetFileAttributes(file, win32con.FILE_ATTRIBUTE_NORMAL)
    except Exception:
        pass

    try:
        with open(file, "w") as f:
            f.write(today.strftime("%Y-%m-%d") + "\n")
            for results in result:
                f.write(str(results[0]).strip() + "\t" + str(results[1]).strip() + "\t" + str(results[2]).strip() + "\t" + str(results[3]).strip() + "\n")
    except Exception:
        pass

    try:
        win32api.SetFileAttributes(file, win32con.FILE_ATTRIBUTE_HIDDEN)
    except Exception:
        pass

    return result

def make_specific_sub_missed_class_table(response,valid_subjectC_code, attendance, cached_yes):
    result = []
    if attendance is not None and cached_yes == 'y':
        result = [attendance[i] for i in range (1, len(attendance)) if attendance[i][0] in valid_subjectC_code]
    for (Date, all_data) in response.items():
        for data in all_data:
            if data is not None and data['code'] in valid_subjectC_code:
                result.append([data['code'], data['Date'], data['subjectName'], data['attended'] + '/' + data['total']])
    return result

def ResponseAttempt(roll, password):

    response = attempt(roll, password)

    if not response:
        click.secho('Invalid Credentials, Sorry, try again', fg='red', bold=True)
        password = getpass.getpass("Password : ")
        response = attempt(roll, password)

    if not response:
        click.secho('Invalid Credentials, Sorry, try one more time.', fg='red', bold=True)
        password = getpass.getpass("Password : ")
        response = attempt(roll, password)

    if not response:
        click.secho('Invalid Credentials, 3 incorrect attempts', fg='red', bold=True)
        exit(0)

    return response,password


def show_table(table,table_headers):
    print(tabulate(table, headers=table_headers,tablefmt="fancy_grid"))







from robobrowser import RoboBrowser

from bs4 import BeautifulSoup
import click
import sys 

def NetworkErrorClick():
    click.echo(click.style('Network error, please check your internet connection and firewall for URL: http://erp.iitbbs.ac.in', fg='red', bold=True))
    exit(0)


def attempt(user, password):
    url = 'http://erp.iitbbs.ac.in'
    browser = RoboBrowser(history=False, parser='html.parser')
    try:
        browser.open(url)
    except:
        NetworkErrorClick()

    try:
        form = browser.get_form(action='login.php')
    except:
        NetworkErrorClick()

    if not form:
        click.echo(click.style('Network error, Unable to fetch form \n', fg='red', bold=True))
        exit(0)

    
    form['email'].value = user
    form['password'].value = password

    try:
        browser.submit_form(form)
    except:
        NetworkErrorClick()

    if (browser.url != 'http://erp.iitbbs.ac.in/home.php'):
        return False

    attendance_link = 'http://erp.iitbbs.ac.in/biometric/list_students.php'

    try:
        browser.open(attendance_link)
    except:
        NetworkErrorClick()

    soup = BeautifulSoup(browser.response.text, 'html.parser')
    content = soup.find('div', attrs={'id': 'content'})
    table = content.find('table')

    tr = table.find_all('tr')
    result = dict()

    for row in tr[1:]: # Don't need headers
        td = row.find_all('td')
        code = td[0].text.strip()
        subject = td[1].text.strip()
        attended_class = td[2].text.strip()
        total_class = td[3].text.strip()
        percentage = td[4].text.strip()
        result[code] = {
            'code' : code,
            'name' : subject,
            'attended': attended_class,
            'total': total_class,
            'percentage': percentage
        }

    return result


from robobrowser import RoboBrowser

from bs4 import BeautifulSoup
import click
import sys 
import datetime
from tabulate import tabulate


def MissedClassDates(user, password, start = "2019-07-19"):
    
    url = 'http://erp.iitbbs.ac.in'
    browser = RoboBrowser(history=False, parser='html.parser')
    try:
        browser.open(url)
    except:
        click.echo(click.style('Network error, please check your internet connection and firewall for URL: http://erp.iitbbs.ac.in \n', fg='red', bold=True))
        exit(0) 

    try:
        form = browser.get_form(action='login.php')
    except:
        click.echo(click.style('Network error, please check your internet connection and firewall for URL: http://erp.iitbbs.ac.in \n', fg='red', bold=True))
        exit(0)

    if not form:
        click.echo(click.style('Network error, Unable to fetch form \n', fg='red', bold=True))
        exit(0)

    
    form['email'].value = user
    form['password'].value = password

    try:
        browser.submit_form(form)
    except:
        click.echo(click.style('Network error, please check your internet connection and firewall for URL: http://erp.iitbbs.ac.in \n', fg='red', bold=True))
        exit(0)

    if (browser.url != 'http://erp.iitbbs.ac.in/home.php'):
        return False

    attendance_link = 'http://erp.iitbbs.ac.in/biometric/list_students_date_wise.php'

    try:
        browser.open(attendance_link)
    except:
        click.echo(click.style('Network error, please check your internet connection and firewall for URL: http://erp.iitbbs.ac.in \n', fg='red', bold=True))
        exit(0)

    today = datetime.date.today() 

    start_date = start #start of semester 
    end_date = today.strftime("%Y-%m-%d")
    st = RangeOfDate(start_date, end_date)
    if len(st) == 0:
        return dict()
    StartdateIndex = 0
    EnddateIndex = len(st) - 1
    return ResponseTable(st, StartdateIndex, EnddateIndex,browser)

def RangeOfDate(startDate, EndDate):

    startDate = startDate.strip()
    start = datetime.datetime.strptime(startDate, "%Y-%m-%d")
    end = datetime.datetime.strptime(EndDate, "%Y-%m-%d")
    if start >= end:
        return list()
    date_generated = [start + datetime.timedelta(days=x) for x in range(0, (end-start).days+1) if (start + datetime.timedelta(days=x)).weekday() != 6]
    DateRange = [date.strftime("%Y-%m-%d") for date in date_generated]
    return DateRange

def ResponseTable(Dates, StartdateIndex, EnddateIndex, browser):
    result = dict()
    Startdate = Dates[StartdateIndex]
    Enddate = Dates[EnddateIndex]

    try:
        Date_form = browser.get_form(action='list_students_date_wise.php')
    except:
        click.echo(click.style('Network error, please check your internet connection and firewall for URL: http://erp.iitbbs.ac.in \n', fg='red', bold=True))
        exit(0)

    Date_form['from'] = Startdate
    Date_form['to'] = Enddate

    try:
        browser.submit_form(Date_form)
    except:
        click.echo(click.style('Network error, please check your internet connection and firewall for URL: http://erp.iitbbs.ac.in \n', fg='red', bold=True))
        exit(0)

    Date_soup = BeautifulSoup(browser.response.text, 'html.parser')
    table = Date_soup.find('table', attrs={'border': '1'})

    tr = table.find_all('tr')
    if(Startdate != Enddate):
        total_class_attended = 0
        total_class_conducted = 0
        for row in tr[1:]:
            td = row.find_all('td')
            total_class_attended += int(td[2].text.strip())
            total_class_conducted += int(td[3].text.strip())
        if(total_class_attended != total_class_conducted):
            MidwayIndex = (StartdateIndex + EnddateIndex)//2
            result.update(ResponseTable(Dates, StartdateIndex, MidwayIndex, browser))
            result.update(ResponseTable(Dates, MidwayIndex+1, EnddateIndex, browser))
    else:
        result[Startdate] = list()
        for row in tr[1:]: # Don't need headers
            td = row.find_all('td')
            code = td[0].text.strip()
            subject = td[1].text.strip()
            attended_class = td[2].text.strip()
            total_class = td[3].text.strip()
            percentage = td[4].text.strip()
            if(total_class != attended_class):
                result[Startdate].append({
                'Date' :  Startdate,
                'code' : code,
                'subjectName' : subject,
                'attended': attended_class,
                'total': total_class,
                'percentage': percentage
                })

    return result